# -*- coding: utf-8 -*-

from odoo import models


class AccountMove(models.Model):
    _inherit = 'account.move'

    def _get_paid_payment_states(self):
        """
        This function returns all the payment_state we can have when a payment is registered.
        It can either be an outstanding payment either a regular payment.
        """
        return ['in_payment', 'paid']

    def _invoice_paid_hook(self):
        res = super()._invoice_paid_hook()
        self._purchase_service_generation()
        return res

    def _purchase_service_generation(self):
        """
        This function is meant to add/cancel a line in a purchase order when we invoice the sale of a product which is 'service_to_purchase'.
        It will:
            #1. For each record, select the invoices that are - related to a sale order ('out_invoice' 'out_refund')
                                                              - not confirmed yet,
                                                              - contain at least one product 'service_to_purchase'.
            #2. select the source invoice and ensure it is confirmed and from the right type.
            #3. For each invoice line, Go back to the sale_order_line linked and update the purchase order from that line to have the right
                reference on the PO line.

            WARNING : If there are several lines with the same product, and the source invoice is linked to several sale orders (which should not happen)
                      then the reference that we put on the purchase_order_line of the second and following lines will match the wrong source document.
        """
        invoices = self.filtered(
            lambda i: i.move_type in ['out_refund', 'out_invoice']
            and any(i.invoice_line_ids.mapped('product_id.service_to_purchase'))
            and i.state == 'posted'
            and i.payment_state in self._get_paid_payment_states()
        )  # 1

        for invoice in invoices:
            source_invoice = invoice.reversed_entry_id if invoice.move_type == 'out_refund' else invoice
            if source_invoice.move_type == "out_invoice" and source_invoice.state == 'posted':  # 2
                for line in invoice.invoice_line_ids.filtered(lambda l: l.product_id.service_to_purchase):
                    source_line = source_invoice.invoice_line_ids.filtered(lambda l: l.product_id == line.product_id)  # noqa: B023
                    # find the sale order line related to the source invoice line and update the po from there
                    so_line = source_line.sale_line_ids.filtered(
                        lambda s: s.order_id.name == source_invoice.invoice_origin  # noqa: B023
                    )[-1:]
                    if so_line:
                        so_line._create_or_update_po_lines(subtract=invoice.move_type == 'out_refund', qty=line.quantity)  # 3
